@@ -50,9 +50,7 @@ from khmer._khmer import Nodegraph as _Nodegraph
 from khmer._khmer import Nodetable as _Nodetable
 from khmer._khmer import HLLCounter as _HLLCounter
 from khmer._khmer import ReadAligner as _ReadAligner
-from khmer._khmer import LinearAssembler
-from khmer._khmer import SimpleLabeledAssembler
-from khmer._khmer import JunctionCountAssembler
+
 from khmer._khmer import HashSet
 from khmer._khmer import Read
 from khmer._khmer import forward_hash
@@ -515,3 +513,6 @@ class ReadAligner(_ReadAligner):
         the traditional way.
         """
         _ReadAligner.__init__(self)
+
+from khmer._oxli.assembly import (LinearAssembler, SimpleLabeledAssembler,
+                                  JunctionCountAssembler)
