@@ -1,31 +1,62 @@
+# This file is part of khmer, https://github.com/dib-lab/khmer/, and is
+# Copyright (C) 2010-2015, Michigan State University.
+# Copyright (C) 2015-2016, The Regents of the University of California.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
+#
+#     * Redistributions in binary form must reproduce the above
+#       copyright notice, this list of conditions and the following
+#       disclaimer in the documentation and/or other materials provided
+#       with the distribution.
+#
+#     * Neither the name of the Michigan State University nor the names
+#       of its contributors may be used to endorse or promote products
+#       derived from this software without specific prior written
+#       permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# This file is part of khmer, http://github.com/ged-lab/khmer/, and is
-# Copyright (C) Michigan State University, 2009-2013. It is licensed under
-# the three-clause BSD license; see doc/LICENSE.txt.
 # Contact: khmer-project@idyll.org
-#
+# pylint: disable=C0111,C0103,missing-docstring,no-member,protected-access
 
-# pylint: disable=C0111,C0103
+from __future__ import print_function
+from __future__ import absolute_import
 
 import khmer
-import khmer_tst_utils as utils
-from nose.plugins.attrib import attr
+
+import pytest
+from . import khmer_tst_utils as utils
+
 
 MAX_COUNT = 255
 
 
-def test_no_collision():
-    kh = khmer.new_hashtable(4, 4)
-
-    kh.count('AAAA')
-    assert kh.get('AAAA') == 1
-
-    kh.count('TTTT')                    # reverse complement
-    assert kh.get('TTTT') == 2
+@pytest.mark.huge
+def test_toobig():
+    try:
+        khmer.Countgraph(4, 1000000000000, 1)
+        assert 0, "this should fail"
+    except MemoryError as err:
+        print(str(err))
 
 
 def test_collision():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
     kh.count('AAAA')
     assert kh.get('AAAA') == 1
@@ -35,32 +66,25 @@ def test_collision():
 
 
 def test_badcount():
-    countingtable = khmer.new_hashtable(4, 4)
+    countgraph = khmer._Countgraph(4, [5])
     try:
-        countingtable.count()
+        countgraph.count()
         assert 0, "count should require one argument"
     except TypeError as err:
-        print str(err)
+        print(str(err))
     try:
-        countingtable.count('ABCDE')
+        countgraph.count('ABCDE')
         assert 0, "count should require k-mer size to be equal"
     except ValueError as err:
-        print str(err)
-
-
-def test_hashtable_n_entries():
-    countingtable = khmer.new_hashtable(4, 4)
-    try:
-        countingtable.n_entries("nope")
-        assert 0, "n_entries should accept no arguments"
-    except TypeError as err:
-        print str(err)
+        print(str(err))
 
 
 def test_complete_no_collision():
-    kh = khmer.new_hashtable(4, 4 ** 2)
+    kh = khmer._Countgraph(4, [4 ** 4])
 
-    for i in range(0, kh.n_entries()):
+    n_entries = kh.hashsizes()[0]
+
+    for i in range(0, n_entries):
         s = khmer.reverse_hash(i, 4)
         kh.count(s)
 
@@ -68,7 +92,7 @@ def test_complete_no_collision():
     n_rc_filled = 0
     n_fwd_filled = 0
 
-    for i in range(0, kh.n_entries()):
+    for i in range(0, n_entries):
         s = khmer.reverse_hash(i, 4)
         if kh.get(s):                   # string hashing is rc aware
             n_rc_filled += 1
@@ -77,16 +101,17 @@ def test_complete_no_collision():
         if kh.get(i):                   # int hashing is not rc aware
             n_fwd_filled += 1
 
-    assert n_rc_filled == kh.n_entries(), n_rc_filled
-    assert n_palindromes == 16, n_palindromes  # @CTB check this
-    assert n_fwd_filled == kh.n_entries() / 2 + n_palindromes / 2, \
-        n_fwd_filled
+    assert n_rc_filled == n_entries, n_rc_filled
+    assert n_palindromes == 16, n_palindromes
+    assert n_fwd_filled == n_entries // 2 + n_palindromes // 2, \
+        (n_fwd_filled, n_entries // 2 + n_palindromes // 2)
 
 
 def test_complete_2_collision():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
-    for i in range(0, kh.n_entries()):
+    n_entries = kh.hashsizes()[0]
+    for i in range(0, n_entries):
         s = khmer.reverse_hash(i, 4)
         kh.count(s)
 
@@ -101,13 +126,14 @@ def test_complete_2_collision():
     #        n_fwd_filled += 1
 
     assert n_rc_filled == 128, n_rc_filled
-    # @CTB assert n_fwd_filled == 100 # kt.n_entries() / 2, n_fwd_filled
 
 
 def test_complete_4_collision():
-    kh = khmer.new_hashtable(4, 2)
+    kh = khmer._Countgraph(4, [3])
 
-    for i in range(0, kh.n_entries()):
+    n_entries = kh.hashsizes()[0]
+
+    for i in range(0, n_entries):
         s = khmer.reverse_hash(i, 4)
         kh.count(s)
 
@@ -122,19 +148,18 @@ def test_complete_4_collision():
     #       n_fwd_filled += 1
 
     assert n_rc_filled == 64, n_rc_filled
-    # @CTB assert n_fwd_filled == kt.n_entries() / 2, n_fwd_filled
 
 
 def test_maxcount():
     # hashtable should saturate at some point so as not to overflow counter
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
     last_count = None
     for _ in range(0, 10000):
         kh.count('AAAA')
         c = kh.get('AAAA')
 
-        print last_count, c
+        print(last_count, c)
         if c == last_count:
             break
         last_count = c
@@ -145,7 +170,7 @@ def test_maxcount():
 
 def test_maxcount_with_bigcount():
     # hashtable should not saturate, if use_bigcount is set.
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
     kh.set_use_bigcount(True)
 
     last_count = None
@@ -153,7 +178,7 @@ def test_maxcount_with_bigcount():
         kh.count('AAAA')
         c = kh.get('AAAA')
 
-        print last_count, c
+        print(last_count, c)
         if c == last_count:
             break
         last_count = c
@@ -163,7 +188,7 @@ def test_maxcount_with_bigcount():
 
 
 def test_consume_uniqify_first():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
     s = "TTTT"
     s_rc = "AAAA"
@@ -175,7 +200,7 @@ def test_consume_uniqify_first():
 
 def test_maxcount_consume():
     # hashtable should saturate at some point so as not to overflow counter
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
     s = "A" * 10000
     kh.consume(s)
@@ -186,7 +211,7 @@ def test_maxcount_consume():
 
 def test_maxcount_consume_with_bigcount():
     # use the bigcount hack to avoid saturating the hashtable.
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
     kh.set_use_bigcount(True)
 
     s = "A" * 10000
@@ -197,21 +222,21 @@ def test_maxcount_consume_with_bigcount():
 
 
 def test_get_mincount():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
     s = "AAAAACGT"
     kh.consume(s)
 
     x = kh.get_min_count(s)
-    assert x == 1
+    assert x == 1, x
 
     kh.consume(s)
     x = kh.get_min_count(s)
-    assert x == 2
+    assert x == 2, x
 
 
 def test_get_maxcount():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [7])
 
     s = "AAAAACGT"
     kh.consume(s)
@@ -225,29 +250,29 @@ def test_get_maxcount():
 
 
 def test_get_maxcount_rc():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [7])
 
     s = "AAAAACGT"
     src = "ACGTTTTT"
     kh.consume(s)
 
     x = kh.get_max_count(s)
-    assert x == 2
+    assert x == 2, x
 
     kh.consume(src)
     x = kh.get_max_count(s)
-    assert x == 4
+    assert x == 4, x
 
 
 def test_get_mincount_rc():
-    kh = khmer.new_hashtable(4, 4)
+    kh = khmer._Countgraph(4, [5])
 
     s = "AAAAACGT"
     src = "ACGTTTTT"
 
     kh.consume(s)
     x = kh.get_min_count(s)
-    assert x == 1
+    assert x == 1, x
 
     kh.consume(src)
     x = kh.get_min_count(s)
@@ -255,7 +280,7 @@ def test_get_mincount_rc():
 
 
 def test_badget():
-    kh = khmer.new_hashtable(6, 4 ** 10)
+    kh = khmer.Countgraph(6, 4 ** 10, 1)
 
     DNA = "AGCTTTTCATTCTGACTGCAACGGGCAATATGTCTCTGTGTGGATTAAAAAAAGAGTGTCTGATAG"
 
@@ -269,11 +294,11 @@ def test_badget():
         kh.get("AGCTT")
         assert 0, "this should fail"
     except ValueError as err:
-        print str(err)
+        print(str(err))
 
 
 def test_64bitshift():
-    kh = khmer.new_hashtable(25, 4)
+    kh = khmer.Countgraph(25, 4, 1)
     fullstr = "GTATGCCAGCTCCAACTGGGCCGGTACGAGCAGGCCATTGCCTCTTGCCGCGATGCGTCGGCG"
     substr = "ATGCCAGCTCCAACTGGGCCGGTACGAGCAGGCCATTGCCTCTTGC"
 
@@ -282,7 +307,7 @@ def test_64bitshift():
 
 
 def test_64bitshift_2():
-    kh = khmer.new_hashtable(25, 4)
+    kh = khmer.Countgraph(25, 4, 1)
     fullstr = "GTATGCCAGCTCCAACTGGGCCGGTACGAGCAGGCCATTGCCTCTTGCCGCGATGCGTCGGCG"
 
     kh.consume(fullstr)
@@ -293,13 +318,13 @@ def test_64bitshift_2():
 
 def test_very_short_read():
     short_filename = utils.get_test_data('test-short.fa')
-    kh = khmer.new_hashtable(9, 4)
-    n_reads, n_kmers = kh.consume_fasta(short_filename)
+    kh = khmer.Countgraph(9, 4, 1)
+    n_reads, n_kmers = kh.consume_seqfile(short_filename)
     assert n_reads == 1, n_reads
     assert n_kmers == 0, n_kmers
 
-    kh = khmer.new_hashtable(8, 4)
-    n_reads, n_kmers = kh.consume_fasta(short_filename)
+    kh = khmer.Countgraph(8, 4, 1)
+    n_reads, n_kmers = kh.consume_seqfile(short_filename)
     assert n_reads == 1, n_reads
     assert n_kmers == 1, n_kmers
 
@@ -307,7 +332,7 @@ def test_very_short_read():
 class Test_ConsumeString(object):
 
     def setup(self):
-        self.kh = khmer.new_hashtable(4, 4 ** 4)
+        self.kh = khmer._Countgraph(4, [4 ** 4])
 
     def test_n_occupied(self):
         assert self.kh.n_occupied() == 0
@@ -319,58 +344,7 @@ class Test_ConsumeString(object):
             self.kh.n_occupied("MU", 1, 3)
             assert 0, "n_occupied shouldn't accept three arguments"
         except TypeError as err:
-            print str(err)
-
-    def test_abundance_by_pos(self):
-        kh = self.kh
-
-        for _ in range(0, 300):
-            kh.count('ATCG')
-
-        for _ in range(0, 10):
-            kh.count('ATGG')
-
-        short_filename = utils.get_test_data('test-short.fa')
-        dist = kh.fasta_count_kmers_by_position(short_filename, 6, 10)
-        assert dist[4] == 1
-        assert sum(dist) == 1
-
-        dist = kh.fasta_count_kmers_by_position(short_filename, 6, MAX_COUNT)
-        assert dist[0] == 1, dist[0]
-        assert dist[2] == 1
-        assert sum(dist) == 2
-
-    def test_abundance_by_pos_bigcount(self):
-        kh = self.kh
-        kh.set_use_bigcount(True)       # count past MAX_COUNT
-
-        for _ in range(0, 300):
-            kh.count('ATCG')
-
-        for _ in range(0, 10):
-            kh.count('ATGG')
-
-        short_filename = utils.get_test_data('test-short.fa')
-        dist = kh.fasta_count_kmers_by_position(short_filename, 6, 10)
-        assert dist[4] == 1
-        assert sum(dist) == 1
-
-        dist = kh.fasta_count_kmers_by_position(short_filename, 6, 300)
-        assert dist[0] == 1, dist[0]
-        assert dist[2] == 1
-        assert sum(dist) == 2
-
-    def test_n_occupied_args(self):
-        assert self.kh.n_occupied() == 0
-        self.kh.consume('AAAA')
-        assert self.kh.n_occupied(0, 1) == 1
-        assert self.kh.n_occupied(1, 4 ** 4) == 0
-
-        hashvalue = khmer.forward_hash('AACT', 4)
-        self.kh.consume('AACT')
-        assert self.kh.n_occupied(0, hashvalue + 1) == 2
-        assert self.kh.n_occupied(hashvalue + 1, 4 ** 4) == 0
-        assert self.kh.n_occupied(hashvalue, hashvalue + 1) == 1
+            print(str(err))
 
     def test_simple(self):
         n = self.kh.consume('AAAA')
@@ -403,14 +377,14 @@ class Test_ConsumeString(object):
 class Test_AbundanceDistribution(object):
 
     def setup(self):
-        self.kh = khmer.new_hashtable(4, 4)
+        self.kh = khmer._Countgraph(4, [5])
         A_filename = utils.get_test_data('all-A.fa')
-        self.kh.consume_fasta(A_filename)
+        self.kh.consume_seqfile(A_filename)
 
     def test_count_A(self):
         A_filename = utils.get_test_data('all-A.fa')
 
-        tracking = khmer.new_hashbits(4, 4, 1)
+        tracking = khmer._Nodegraph(4, [5])
         dist = self.kh.abundance_distribution(A_filename, tracking)
 
         assert sum(dist) == 1
